@@ -1,5 +1,5 @@
 /* preferences.cc -- preferences dialog for falling blocks game.  -*- C++ -*-
-   Copyright (C) 2011-2012 Gerardo Ballabio
+   Copyright (C) 2011-2013 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,34 +88,37 @@ bool PrefsDialog::on_key_press_event(GdkEventKey *event)
 
 void PrefsDialog::on_geometry_radio_signal_toggled()
 {
-  // square, hexagonal, triangular, cubic
-  bool is_3d = get_geometry() == 3;
-
-  depth_box.set_sensitive(is_3d);
-  if (is_3d)
+  if (get_geometry_3d())
     {
+      depth_box.set_sensitive(true);
+
       const int min = 4, max = 8;
       width_adjust.set_lower(min);
       width_adjust.set_upper(max);
-      if (get_width() > max)
-	width_spin.set_value(max);
+      width_spin.set_value(width3);
     }
   else
     {
+      depth_box.set_sensitive(false);
+
       const int min = 10, max = 20;
       width_adjust.set_lower(min);
       width_adjust.set_upper(max);
-      if (get_width() < min)
-	width_spin.set_value(min);
+      width_spin.set_value(width2);
     }
 }
 
 void PrefsDialog::on_width_spin_signal_value_changed()
 {
-  int max = get_width();
-  depth_adjust.set_upper(max);
-  if (get_depth() > max)
-    depth_spin.set_value(max);
+  if (get_geometry_3d())
+    {
+      width3 = get_width();
+      depth_adjust.set_upper(width3);
+      if (get_depth() > width3)
+	depth_spin.set_value(width3);
+    }
+  else
+    width2 = get_width();
 }
 
 void PrefsDialog::on_mode_combo1_signal_changed()
@@ -149,7 +152,7 @@ PrefsDialog::PrefsDialog()
   : Gtk::Dialog("Preferences", true), // modal dialog
     width_adjust(0, 0, 0), depth_adjust(0, 0, 0), speed_adjust(0, 0, 0),
     bricks_adjust1(0, 0, 0), bricks_adjust2(0, 0, 0), empty_adjust(0, 0, 0),
-    edit_keys(0)
+    width2(10), width3(6), edit_keys(0)
 {
   get_vbox()->pack_start(notebook);
   notebook.set_border_width(10);
@@ -173,7 +176,7 @@ PrefsDialog::PrefsDialog()
   for (unsigned i=0; i<sizeof(geometry_radio)/sizeof(geometry_radio[0]); ++i)
     {
       geometry_radio[i].set_group(geometry_group);
-      geometry_radio[i].set_label(brick_shapes[i]);
+      geometry_radio[i].set_label(brick_shape[i].description);
       geometry_box2.pack_start(geometry_radio[i]);
       geometry_radio[i].signal_toggled().connect
 	(sigc::mem_fun(*this,
@@ -200,8 +203,8 @@ PrefsDialog::PrefsDialog()
   depth_box.pack_start(depth_spin, Gtk::PACK_SHRINK);
   depth_spin.set_adjustment(depth_adjust);
   depth_adjust.set_lower(4);
-  depth_adjust.set_upper(10);
-  depth_adjust.set_value(4);
+  depth_adjust.set_upper(width3);
+  depth_adjust.set_value(width3);
   depth_box.set_sensitive(false);
 
   tab_game.pack_start(level_box1, Gtk::PACK_SHRINK);
@@ -314,7 +317,8 @@ PrefsDialog::PrefsDialog()
   two_tree.append_column("Action", two_columns.action);
   two_tree.append_column_editable("Key", two_columns.key);
   two_tree.set_headers_visible(false);
-  for (int i=0; i<sizeof(control_keys_2d)/sizeof(control_keys_2d[0]); ++i)
+  for (unsigned i=0;
+       i<sizeof(control_keys_2d)/sizeof(control_keys_2d[0]); ++i)
     {
       Gtk::TreeModel::Row row = *(two_list->append());
       row[two_columns.action] = control_keys_2d[i].action;
@@ -335,7 +339,8 @@ PrefsDialog::PrefsDialog()
   three_tree.append_column("Action", three_columns.action);
   three_tree.append_column_editable("Key", three_columns.key);
   three_tree.set_headers_visible(false);
-  for (int i=0; i<sizeof(control_keys_3d)/sizeof(control_keys_3d[0]); ++i)
+  for (unsigned i=0;
+       i<sizeof(control_keys_3d)/sizeof(control_keys_3d[0]); ++i)
     {
       Gtk::TreeModel::Row row = *(three_list->append());
       row[three_columns.action] = control_keys_3d[i].action;
@@ -363,6 +368,11 @@ unsigned PrefsDialog::get_geometry() const
   return 0;
 }
 
+bool PrefsDialog::get_geometry_3d() const
+{
+  return brick_shape[get_geometry()].is_3d;
+}
+
 unsigned PrefsDialog::get_key_2d(int n) const
 {
   Gtk::TreeModel::iterator iter = two_list->children().begin();
@@ -505,8 +515,18 @@ bool PrefsDialog::load()
 	      else if (name == "reflection")
 		geometry_check.set_active
 		  (std::atoi(reader->get_value().c_str()));
-	      else if (name == "width")
-		width_spin.set_value(std::atoi(reader->get_value().c_str()));
+	      else if (name == "width2")
+		{
+		  width2 = std::atoi(reader->get_value().c_str());
+		  if (!get_geometry_3d())
+		    width_spin.set_value(width2);
+		}
+	      else if (name == "width3")
+		{
+		  width3 = std::atoi(reader->get_value().c_str());
+		  if (get_geometry_3d())
+		    width_spin.set_value(width3);
+		}
 	      else if (name == "depth")
 		depth_spin.set_value(std::atoi(reader->get_value().c_str()));
 	      else if (name == "mode1")
@@ -555,7 +575,8 @@ bool PrefsDialog::save() const
 
   os << "  <geometry value=\"" << get_geometry() << "\" />" << std::endl;
   os << "  <reflection value=\"" << get_reflection() << "\" />" << std::endl;
-  os << "  <width value=\"" << get_width() << "\" />" << std::endl;
+  os << "  <width2 value=\"" << width2 << "\" />" << std::endl;
+  os << "  <width3 value=\"" << width3 << "\" />" << std::endl;
   os << "  <depth value=\"" << get_depth() << "\" />" << std::endl;
   os << "  <mode1 value=\"" << get_mode1() << "\" />" << std::endl;
   os << "  <mode2 value=\"" << get_mode2() << "\" />" << std::endl;
